package armotypes

type VulnerabilityJobParams struct {
	Name            string `json:"name,omitempty"`
	ID              string `json:"id,omitempty"`
	ClusterName     string `json:"clusterName"`
	Namespace       string `json:"namespace"`
	CronTabSchedule string `json:"cronTabSchedule,omitempty"`
	JobID           string `json:"jobID,omitempty"`
}

type Vulnerability struct {
	Name               string                         `json:"name" bson:"name"`
	ImgHash            string                         `json:"imageHash" bson:"imageHash"`
	ImgTag             string                         `json:"imageTag" bson:"imageTag"`
	RelatedPackageName string                         `json:"packageName" bson:"packageName"`
	PackageVersion     string                         `json:"packageVersion" bson:"packageVersion"`
	Link               string                         `json:"link" bson:"link"`
	Description        string                         `json:"description" bson:"description"`
	Severity           string                         `json:"severity" bson:"severity"`
	SeverityScore      int                            `json:"severityScore" bson:"severityScore"`
	Metadata           interface{}                    `json:"metadata" bson:"metadata"`
	Fixes              VulFixes                       `json:"fixedIn" bson:"fixedIn"`
	Relevancy          string                         `json:"relevant" bson:"relevant"`
	UrgentCount        int                            `json:"urgent" bson:"urgent"`
	NeglectedCount     int                            `json:"neglected" bson:"neglected"`
	HealthStatus       string                         `json:"healthStatus" bson:"healthStatus"`
	Categories         VulnerabilityCategory          `json:"categories" bson:"categories"`
	ExceptionApplied   []VulnerabilityExceptionPolicy `json:"exceptionApplied,omitempty" bson:"exceptionApplied,omitempty"`
}

type VulFixes []FixedIn

type VulnerabilityCategory struct {
	IsRCE bool `json:"isRce" bson:"isRce"`
}

type FixedIn struct {
	Name    string `json:"name" bson:"name"`
	ImgTag  string `json:"imageTag" bson:"imageTag"`
	Version string `json:"version" bson:"version"`
}
type TopVulItem struct {
	Vulnerability   `json:",inline"`
	WorkloadsCount  int64 `json:"workloadsCount" bson:"workloadsCount"`
	SeverityOverall int64 `json:"severityOverall" bson:"severityOverall"`
}

type CommonContainerScanSummaryResult struct {
	SeverityStats
	Designators     PortalDesignator `json:"designators" bson:"designators"`
	Context         []ArmoContext    `json:"context" bson:"context"`
	JobIDs          []string         `json:"jobIDs" bson:"jobIDs"`
	CustomerGUID    string           `json:"customerGUID" bson:"customerGUID"`
	ContainerScanID string           `json:"containersScanID" bson:"containersScanID"`

	Timestamp     int64    `json:"timestamp" bson:"timestamp"`
	WLID          string   `json:"wlid" bson:"wlid"`
	ImgTag        string   `json:"imageTag" bson:"imageTag"`
	ImgHash       string   `json:"imageHash" bson:"imageHash"`
	Cluster       string   `json:"cluster" bson:"cluster"`
	Namespace     string   `json:"namespace" bson:"namespace"`
	ContainerName string   `json:"containerName" bson:"containerName"`
	PackagesName  []string `json:"packages" bson:"packages"`

	ListOfDangerousArtifcats []string `json:"listOfDangerousArtifcats" bson:"listOfDangerousArtifcats"`

	Status string `json:"status" bson:"status"`

	Registry     string `json:"registry" bson:"registry"`
	VersionImage string `json:"versionImage" bson:"versionImage"`

	SeveritiesStats         []SeverityStats `json:"severitiesStats" bson:"severitiesStats"`
	ExcludedSeveritiesStats []SeverityStats `json:"excludedSeveritiesStats,omitempty" bson:"excludedSeveritiesStats,omitempty"`

	Version string `json:"version" bson:"version"`

	Vulnerabilities []ShortVulnerabilityResult `json:"vulnerabilities" bson:"vulnerabilities"`
}

type SeverityStats struct {
	Severity                     string `json:"severity,omitempty" bson:"severity,omitempty"`
	TotalCount                   int64  `json:"total" bson:"total"`
	RCEFixCount                  int64  `json:"rceFixCount" bson:"rceFixCount"`
	FixAvailableOfTotalCount     int64  `json:"fixedTotal" bson:"fixedTotal"`
	RelevantCount                int64  `json:"totalRelevant" bson:"totalRelevant"`
	FixAvailableForRelevantCount int64  `json:"fixedRelevant" bson:"fixedRelevant"`
	RCECount                     int64  `json:"rceTotal" bson:"rceTotal"`
	UrgentCount                  int64  `json:"urgent" bson:"urgent"`
	NeglectedCount               int64  `json:"neglected" bson:"neglected"`
	HealthStatus                 string `json:"healthStatus" bson:"healthStatus"`
}

type ShortVulnerabilityResult struct {
	Name string `json:"name" bson:"name"`
}
